#[macro_use]
extern crate rocket;

use poise::serenity_prelude::Http;
use rocket::fs::{relative, FileServer};
use rocket_dyn_templates::{tera, Template};
use sass_rocket_fairing::SassFairing;
use std::{collections::HashMap, env};

mod models;
use models::Settings;

mod utils;

mod cookies;

mod routes;
use routes::*;

mod i18n;
use i18n::{i18n_filter, load_catalogs};

use crate::i18n::langs_filter;

mod prelude;

#[launch]
async fn rocket() -> _ {
    dotenv::dotenv().expect("Failed to load .env file");

    let token = env::var("DISCORD_TOKEN").expect("Expected a token in the environment");
    let http = Http::new(&token);

    let config = rocket::Config::figment().merge(("port", 1313)).merge((
        "secret_key",
        env::var("SECRET")
            .expect("Must have SECRET defined in .env for private cookie persistence"),
    ));

    rocket::custom(config)
        .manage(Settings::new(&http).await.unwrap())
        .manage(http)
        .mount(
            "/",
            routes![get_challenge, get_guilds, login, post_login, success, logout, testing],
        )
        .mount("/css", FileServer::from(relative!("styles/css")))
        .mount("/", FileServer::from(relative!("static")).rank(1))
        .attach(Template::custom(move |engines| {
            use tera::Value;
            let (catalogs, langs) = load_catalogs().unwrap();
            engines.tera.register_filter(
                "i18n",
                move |value: &Value, args: &HashMap<String, Value>| {
                    i18n_filter(value, args, &catalogs)
                },
            );
            engines.tera.register_filter(
                "langs",
                move |value: &Value, args: &HashMap<String, Value>| {
                    langs_filter(value, args, &langs)
                },
            )
        }))
        .attach(SassFairing::default())
}
